package platform_user

import (
	"context"
	"github.com/robfig/cron/v3"
	"github.com/zeromicro/go-zero/core/logx"
	pbMq "live-service/app/pb/mq"
	"live-service/app/user_center/rpc/internal/svc"
)

// PlatformUser 仅提取需要的部分,其余的json后装到p_info
type (
	PlatformUser struct {
		PUid    string
		PName   string
		PAvatar string
		PInfo   string
	}
	UserRetriever struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext

		retrieveMapper map[string]RetrieveFunc

		logx.Logger
	}
)

type RetrieveFunc func(pUid string) (*PlatformUser, error)

func NewUserRetriever(ctx context.Context, svcCtx *svc.ServiceContext) *UserRetriever {
	result := &UserRetriever{
		ctx:    ctx,
		svcCtx: svcCtx,
		retrieveMapper: map[string]RetrieveFunc{
			pbMq.Platform_name[int32(pbMq.Platform_bilibili)]: RetrieveBilibili,
		},
		Logger: logx.WithContext(ctx),
	}

	go result.scheduler()
	return result
}

func (r *UserRetriever) retrieveUser(platform, pUid string) *PlatformUser {
	if handle, ok := r.retrieveMapper[platform]; ok {
		if p, err := handle(pUid); err != nil {
			return p
		} else {
			r.Logger.Errorf("获取平台用户信息错误: err: %s\n", err.Error())
		}
	}
	return &PlatformUser{
		PUid:    pUid,
		PName:   "",
		PAvatar: "",
		PInfo:   "{}", // 空json
	}
}

func (r *UserRetriever) scheduler() {
	c := cron.New()
	_, _ = c.AddFunc("@every 300ms", func() {
		list, err := r.svcCtx.UserPlatformModel.FindUpdatableList(r.ctx, r.svcCtx.Config.User.UpdateDuration, 10)
		if err != nil {
			return
		}
		for _, dbModel := range list {
			pUser := r.retrieveUser(dbModel.Platform, dbModel.PUid)
			if pUser.PInfo == "{}" {
				continue
			}
			dbModel.PUname = pUser.PName
			dbModel.PAvatar = pUser.PAvatar
			dbModel.PInfo = pUser.PInfo
			_ = r.svcCtx.UserPlatformModel.Update(r.ctx, &dbModel)
		}
	})
	c.Start()
}
